import type { Message } from '@tencentcloud/chat-react-native';
import type { IMessageModel } from '../interface/model';
import type { GetGroupMessageReadParams } from '../type';
import { StoreName } from '../const';
import type { ITUIChatService } from '../interface/service';

export default class ReadReceiptHandler {
  private TUIChatService: ITUIChatService;
  constructor(TUIChatService: ITUIChatService) {
    this.TUIChatService = TUIChatService;
  }

  private getEngine() {
    return this.TUIChatService.getEngine();
  }

  init() {
    const chatEngine = this.getEngine();
    chatEngine.eventCenter.addEvent(chatEngine.EVENT.MESSAGE_READ_RECEIPT_RECEIVED,
      this.onMessageReadReceiptReceived.bind(this));
  }

  private async onMessageReadReceiptReceived(readReceiptInfoList: any[]) {
    const chatEngine = this.getEngine();
    const currentConversationID: string = chatEngine.TUIStore.getData(StoreName.CONV, 'currentConversationID');
    const messageList: Message[] = [];
    readReceiptInfoList.forEach(async (item: any) => {
      const { messageID } = item;
      const message = await chatEngine.chat.findMessage(messageID);
      // 只更新当前会话的已读回执信息
      if (message && (currentConversationID === message.conversationID)) {
        messageList.push(message);
      }
    });
    if (messageList.length > 0) {
      chatEngine.TUIChat.updateMessageList(messageList, 'edit');
    }
  }

  sendMessageReadReceipt(messageList: IMessageModel[]) {
    const list: Message[] = [];
    messageList.forEach((messageModel: IMessageModel) => {
      const message: Message = messageModel.getMessage();
      list.push(message);
    });
    return this.getEngine().chat.sendMessageReadReceipt(list);
  }

  getGroupMessageReadMemberList(options: GetGroupMessageReadParams) {
    const message: Message = options.message.getMessage();
    const params = {
      ...options,
      message,
    };
    return this.getEngine().chat.getGroupMessageReadMemberList(params);
  }

  getMessageReadReceiptList(messageList: Message[]) {
    const chatEngine = this.getEngine();
    const enabledMessageReadReceipt: boolean = chatEngine.TUIStore.getData(StoreName.APP, 'enabledMessageReadReceipt');
    const displayMessageReadReceipt: boolean = chatEngine.TUIStore.getData(StoreName.USER, 'displayMessageReadReceipt');
    // 处理以下case:
    // case1: 业务侧不需要单条已读回执功能
    // case2: UI 层没有开启消息阅读状态展示时不需要获取已读回执信息
    // case3: messageList 为空时直接返回
    // case4: C2C 拉漫游时已经返回了已读回执信息，只需要单独获取群的已读回执信息
    const flag = !enabledMessageReadReceipt
      || !displayMessageReadReceipt
      || (messageList.length === 0)
      || (messageList[0].conversationType !== chatEngine.TYPES.CONV_GROUP);
    if (!flag) {
      chatEngine.chat.getMessageReadReceiptList(messageList).then((imResponse: any) => {
        const { messageList: newMessageList } = imResponse.data;
        this.TUIChatService.updateMessageList(newMessageList, 'edit');
      }).catch((error: any) => {
        console.warn(`ReadReceiptHandler.getMessageReadReceiptList error:${error.message}`);
      });
    }
  }
}
