import type { Profile } from '@tencentcloud/chat-react-native';
import type { UpdateMyProfileParams, UserIDListParams } from '../type';
import { StoreName } from '../const';
import { isUndefined } from '../utils/common-utils';
import type { ITUIUserService } from '../interface/service';

export default class UserProfileHandler {
  private TUIUserService: ITUIUserService;
  constructor(TUIUserService: ITUIUserService) {
    this.TUIUserService = TUIUserService;
  }

  private getEngine() {
    return this.TUIUserService.getEngine();
  }

  init() {
    this.getEngine().eventCenter.addEvent(this.getEngine().EVENT.PROFILE_UPDATED, this.onProfileUpdated.bind(this));
    this.getEngine().eventCenter.addEvent(this.getEngine().EVENT.BLACKLIST_UPDATED, this.onBlacklistUpdated.bind(this));
    this.getUserProfileInitData();
  }

  private onProfileUpdated(userProfileList: Profile[]) {
    const chatEngine = this.getEngine();
    const myProfile = chatEngine.TUIStore.getData(StoreName.USER, 'userProfile');
    userProfileList.forEach((profile: Profile) => {
      // 自己资料的更新通知需要更新 store 并通知给 UI，好友资料更新通知先不处理
      if (profile.userID === myProfile.userID) {
        chatEngine.TUIStore.update(StoreName.USER, 'userProfile', profile);
      }
    });
  }

  private onBlacklistUpdated(blacklist: string[]) {
    const chatEngine = this.getEngine();
    this.getBlacklistProfile(blacklist).then((userBlacklist: any[]) => {
      chatEngine.TUIStore.update(StoreName.USER, 'userBlacklist', userBlacklist);
    });
  }

  private getUserProfileInitData() {
    const chatEngine = this.getEngine();
    if (!chatEngine.chat.isReady()) {
      return;
    }
    chatEngine.chat.getBlacklist().then((imResponse: any) => {
      const blockList = imResponse.data || [];
      console.log(`TUIUserProfileHandler.init, getBlacklist count:${blockList.length}`);
      if (blockList.length > 0) {
        this.onBlacklistUpdated(blockList);
      }
    });
  }

  // 获取黑名单用户资料
  private getBlacklistProfile(blacklist: string[]) {
    // 用户资料一次最大支持查询 100 个用户，需要分组处理
    const blacklistGroup: any[] = [];
    let index = 0;
    while (index < blacklist.length) {
      blacklistGroup.push(blacklist.slice(index, index += 100));
    }
    const promiseList: any[] = [];
    blacklistGroup.forEach((userIDList: string[]) => {
      promiseList.push(
        this.TUIUserService.getUserProfile({ userIDList }),
      );
    });
    const userBlacklist: any[] = [];
    return Promise.all(promiseList).then((imResponseList: any[]) => {
      imResponseList.forEach((imResponse: any) => {
        const filterList: any[] = imResponse.data.map((profile: any) => {
          const { userID, nick, avatar } = profile;
          return { userID, nick, avatar };
        });
        userBlacklist.push(...filterList);
      });
      return userBlacklist;
    });
  }

  public getUserProfile(options?: UserIDListParams) {
    const chatEngine = this.getEngine();
    if (isUndefined(options)) {
      return chatEngine.chat.getMyProfile().then((imResponse: any) => {
        chatEngine.TUIStore.update(StoreName.USER, 'userProfile', imResponse.data);
        return imResponse;
      }).catch((error: any) => Promise.reject(error));
    }
    return chatEngine.chat.getUserProfile(options as UserIDListParams);
  }

  public updateMyProfile(options: UpdateMyProfileParams) {
    return this.getEngine().chat.updateMyProfile(options);
  }

  public addToBlacklist(options: UserIDListParams) {
    return this.getEngine().chat.addToBlacklist(options);
  }

  public removeFromBlacklist(options: UserIDListParams) {
    return this.getEngine().chat.removeFromBlacklist(options);
  }
}
