import type { Conversation, Message } from '@tencentcloud/chat-react-native';
import type TUIBase from '../../tui-base';
import type {
  SendMessageParams,
  GetMessageListParams,
  TranslateTextParams,
  GetGroupMessageReadParams,
  SendMessageOptions,
  SearchCloudMessagesParams,
  GetMessageListHoppingParams,
  SendForwardMessageOptions,
  GetMessageReactionsParams,
  GetAllUserListOfMessageReactionParams,
  ConvertVoiceToTextParams,
} from '../../type';
import type { IMessageHandler } from './message-handler';
import type { IConversationModel, IMessageModel } from '../model';

/**
 * @interface TUIChatService
*/
export interface ITUIChatService extends TUIBase {
  messageHandler: IMessageHandler;

  /**
   * 初始化 Service
   * @function
   * @private
   */
  init: () => void;

  /**
   * 更新 store 里的 messageList
   * @param messageList 要进行更新的 messageList
   * @param type 更新类型
   * @private
   */
  updateMessageList(messageList: Message[], type: string): void;

  /**
   * 发送文本消息
   * @function
   * @param {SendMessageParams} options 文本消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送普通文本消息
   * let promise = TUIChatService.sendTextMessage({
   *  payload: {
   *    text: 'Hello world!'
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 发送普通文本消息并携带消息自定义数据
   * let promise = TUIChatService.sendTextMessage({
   *  payload: {
   *    text: 'Hello world!'
   *  },
   *  cloudCustomData: 'your cloud custom data'
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendTextMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 发送附带 @ 提醒功能的文本消息
   * @function
   * @param {SendMessageParams} options 附带 @ 提醒功能的文本消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送 @ 消息
   * let promise = TUIChatService.sendTextAtMessage({
   *  payload: {
   *    text: '@denny @lucy @所有人 今晚聚餐，收到的请回复1',
   *    atUserList: ['denny', 'lucy', TUIChatEngine.TYPES.MSG_AT_ALL] // 'denny' 'lucy' 都是 userID，而非昵称
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendTextAtMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 发送图片消息
   * @function
   * @param {SendMessageParams} options 图片消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送图片消息
   * let promise = TUIChatService.sendImageMessage({
   *  payload: {
   *    file: file
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendImageMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 发送音频消息
   * @function
   * @param {SendMessageParams} options 音频消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送音频消息
   * let promise = TUIChatService.sendAudioMessage({
   *  payload: {
   *    file: file
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendAudioMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 发送视频消息
   * @function
   * @param {SendMessageParams} options 视频消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送视频消息
   * let promise = TUIChatService.sendVideoMessage({
   *  payload: {
   *    file: file
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendVideoMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 发送文件消息
   * @function
   * @param {SendMessageParams} options 文件消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送文件消息
   * let promise = TUIChatService.sendFileMessage({
   *  payload: {
   *    file: file
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendFileMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 发送自定义消息
   * @function
   * @param {SendMessageParams} options 自定义消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送自定义消息
   * let promise = TUIChatService.sendCustomMessage({
   *  payload: {
   *   data: 'dice', // 用于标识该消息是骰子类型消息
   *   description: String(random(1,6)), // 获取骰子点数
   *   extension: ''
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendCustomMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 发送表情消息
   * @function
   * @param {SendMessageParams} options 表情消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送自定义消息
   * let promise = TUIChatService.sendFaceMessage({
   *  payload: {
   *    index: 1, // Number 表情索引，用户自定义
   *    data: 'tt00' // String 额外数据
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendFaceMessage(
    message: SendMessageParams,
    sendMessageOptions?: SendMessageOptions
  ): Promise<any>;

  /**
   * 发送地理位置消息
   * @function
   * @param {SendMessageParams} options 地理位置消息相关参数
   * @param {SendMessageOptions}[sendMessageOptions] 消息发送选项
   * @example
   * // 发送地理位置消息
   * let promise = TUIChatService.sendLocationMessage({
   *  payload: {
   *   description: '深圳市深南大道10000号腾讯大厦',
   *   longitude: 113.941079, // 经度
   *   latitude: 22.546103 // 纬度
   *  },
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendLocationMessage(
    options: SendMessageParams,
    sendMessageOptions?: SendMessageOptions,
  ): Promise<any>;

  /**
   * 转发消息
   * - 注意：支持逐条转发和合并转发
   * @function
   * @param {Array<Conversation | IConversationModel>} conversationList 接收转发消息的会话列表
   * @param {Array<Message | IMessageModel>} messageList 待转发的消息列表
   * @param {SendForwardMessageOptions}[sendForwardMessageOptions] 消息发送选项
   * @example
   * // 逐条转发，把当前会话的 message1，message2 逐条转发给会话 conversation1, conversation2
   * let promise = TUIChatService.sendForwardMessage([conversation1, conversation2], [message1, message2]);
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 合并转发，把当前会话的 message1，message2 合并转发给会话conversation1, conversation2
   * let promise = TUIChatService.sendForwardMessage([conversation1, conversation2], [message1, message2], { needMerge: true });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 合并转发，把当前会话的 message1，message2 合并转发给会话 conversation1, conversation2, 自定义合并后的消息 title
   * const sendForwardMessageOptions = {
   *    needMerge: true,
   *    // 合并转发配置项，needMerge = true 时生效。
   * `  mergeInfo: {
   *      title: '自定义合并后的消息 title'
   *    }
   * }
   * let promise = TUIChatService.sendForwardMessage([conversation1, conversation2], [message1, message2], sendForwardMessageOptions);
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendForwardMessage(
    conversationList: Conversation[] | IConversationModel[],
    messageList: Message[] | IMessageModel,
    sendForwardMessageOptions?: SendForwardMessageOptions,
  ): Promise<any>;

  /**
   * 撤回单聊消息或者群聊消息。撤回成功后，消息对象的 isRevoked 属性值为 true。
   * @function
   * @param {Message} message 需要撤回的消息对象1
   * @private
   */
  revokeMessage(message: Message): Promise<any>;

  /**
   * 重发消息的接口，当消息发送失败时，可调用该接口进行重发
   * @function
   * @param {Message} message 需要重发的消息对象
   * @private
   */
  resendMessage(message: Message): Promise<any>;

  /**
   * 删除消息的接口。删除成功后，被删除消息的 isDeleted 属性值为 true。
   * - 注意：批量删除多条消息时需要通过调用此 API
   * @function
   * @param {Array<Message>} messageList 需要删除的消息列表
   * @private
   */
  deleteMessage(messageList: Message[]): Promise<any>;

  /**
   * 设置消息扩展。设置成功后，自己和对端用户（C2C）或群组成员（Group）都会收到 MESSAGE_EXTENSIONS_UPDATED 事件。
   * @function
   * @param {Message} message 消息实例
   * @param {Array<object>} extensions 扩展信息
   * @private
   */
  setMessageExtensions(message: Message, extensions: object[]): Promise<any>;

  /**
   * 获取消息扩展。
   * @function
   * @param {Message} message 消息实例
   * @private
   */
  getMessageExtensions(message: Message): Promise<any>;

  /**
   * 删除消息扩展。
   * @function
   * @param {Message} message 消息实例
   * @param {Array<string>|undefined} keyList 扩展信息 key 列表
   * @private
   */
  deleteMessageExtensions(message: Message, keyList?: string[]): Promise<any>;

  /**
   * 变更消息的接口。变更成功后，自己和对端用户（C2C）或群组成员（Group）都会收到 MESSAGE_MODIFIED 事件。
   * @function
   * @param {Message} message 消息实例
   * @private
   */
  modifyMessage(message: Message): Promise<any>;

  /**
   * 引用当前消息
   * @function
   * @param {Message} message 需要被引用的消息实例
   * @private
   */
  quoteMessage(message: Message): Message;

  /**
   * @function
   * @param {Message} message 需要被回复的消息实例
   * @private
   */
  replyMessage(message: Message): Message;

  /**
   * 开始输入
   * - 注意: 只支持 C2C 会话
   * @function
   * @example
   * TUIChatService.enterTypingState();
   */
  enterTypingState(): void;

  /**
   * 结束输入
   * - 注意: 只支持 C2C 会话
   * @function
   * @example
   * TUIChatService.leaveTypingState();
   */
  leaveTypingState(): void;

  /**
   * 发送消息已读回执
   * @function
   * @param {Array<IMessageModel>} messageList messageModel 列表
   * @example
   * let promise = TUIChatService.sendMessageReadReceipt([messageModel1, messageModel2, messageModel3]);
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  sendMessageReadReceipt(messageList: IMessageModel[]): Promise<any>;

  /**
   * 获取群消息已读（或未读）群成员列表
   * @function
   * @param {GetGroupMessageReadParams} options 参数选项
   * @example
   * // 获取已读群成员列表
   * let promise = TUIChatService.getGroupMessageReadMemberList({
   *  message,
   *  filter: 0,
   *  cursor: '', // 第一次拉取传''
   *  count: 30,
   * });
   * promise.then(function(response) {
   *  const { isCompleted, cursor, messageID, readUserInfoList } = response;
   *   // isCompleted - true，拉取完成；false 未完成
   *   // cursor - 当 isCompleted 为 false 的时候用于续拉
   *   // messageID - 群消息的 ID
   *   // readUserInfoList - 已读的群成员用户信息列表
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 获取未读群成员列表
   * let promise = TUIChatService.getGroupMessageReadMemberList({
   *  message,
   *  filter: 1,
   *  cursor: '', // 第一次拉取传''
   *  count: 30,
   * });
   * promise.then(function(response) {
   *  const { isCompleted, cursor, messageID, unreadUserInfoList } = response;
   *   // isCompleted - true，拉取完成；false 未完成
   *   // cursor - 当 isCompleted 为 false 的时候用于续拉
   *   // messageID - 群消息的 ID
   *   // unreadUserInfoList - 未读的群成员用户信息列表
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  getGroupMessageReadMemberList(options: GetGroupMessageReadParams): Promise<any>;

  /**
   * 分页拉取指定会话的消息列表的接口，当用户进入会话首次渲染消息列表或者用户“下拉查看更多消息”时，需调用该接口。
   * @function
   * @param {GetMessageListParams} [options] 获取漫游消息参数
   * @example
   * let promise = TUIChatService.getMessageList();
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  getMessageList(options?: GetMessageListParams): Promise<any>;

  /**
   * 根据指定的消息 sequence 或 消息时间拉取会话的消息列表
   * @function
   * @param {GetMessageListHoppingParams} options 消息实例
   * @private
   */
  getMessageListHopping(options?: GetMessageListHoppingParams): Promise<any>;

  /**
   * 下载合并后的消息。如果发送方发送的合并消息较大，Chat SDK 会将此消息存储到云端，消息接收方查看消息时，需要先把消息从云端下载到本地。
   * @function
   * @param {Record<string, any>} message 消息对象或合并消息对象的 messageBody
   * @example
   * if (message.type === TUIChatEngine.TYPES.MSG_MERGER && message.payload.downloadKey !== '') {
   *  // 下载成功后，SDK会更新 message.payload.messageList 等信息
   *  let promise = TUIChatService.downloadMergedMessages(message);
   *  promise.catch((error) => {
   *    // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   *  });
   * }
   */
  downloadMergedMessages(message: any): Promise<any>;

  /**
   * 设置文本消息翻译的目标语言
   * >注意：v2.1.0 起支持。
   * @function
   * @param {string} language 目标语言，支持的目标语言有多种，例如：英语-"en"，简体中文-"zh"，法语-"fr"，德语-"de"等。
   * @example
   * TUIChatService.setTranslationLanguage('en');
   */
  setTranslationLanguage(language: string): void;

  /**
   * 翻译文本消息
   * @function
   * @param {TranslateTextParams} options 需要翻译的消息及配置信息
   * @example
   * let promise = TUIChatService.translateText({
   *  sourceTextList: ['مرحبًا'],
   *  sourceLanguage: 'auto',
   * });
   * promise.then((chatResponse) => {
   *  // 翻译成功 translatedTextList 和 sourceTextList 的元素一一对应
   *  const { translatedTextList } = chatResponse.data;
   * });
   */
  translateText(options: TranslateTextParams): Promise<any>;

  /**
   * 搜索云端消息
   * >【v0.1.2起支持】<br/>
   * >注意1：此功能当前处于限免阶段，您可通过[即时通信 IM 功能及购买咨询交流群](https://zhiliao.qq.com/s/c5GY7HIM62CK/cXRDBIIM62CT)联系我们为您开通体验完整功能。<br/>
   * >注意2：此接口本地限频 2 次/秒。<br/>
   * >注意3：搜索【全部会话】的消息，如果匹配到的消息数量 messageCount > 1，则接口返回的 messageList 为 []，您可以在 UI 上展示 【${messageCount} 条相关记录】。如果您想高亮展示匹配到的消息，请参考【指定搜索】，将接口返回的 messageList 高亮展示。<br/>
   * >注意4：搜索【全部会话】的消息，如果某个会话中匹配到的消息条数 = 1，则 messageList 为匹配到的那条消息。<br/>
   * >注意5：社群、topic、直播群，不支持搜索云端消息。<br/>
   * @function
   * @param {SearchCloudMessagesParams} options 需要搜索的消息的参数
   * @example
   * // 全量搜索，指定关键字
   * // - 搜索消息里出现 ‘你好’ 或 ‘在哪里’
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   * });
   *
   * @example
   * // 全量搜索，指定关键字和匹配类型搜索
   * // - 搜索消息里出现了 ‘你好’ 与 ‘在哪里’ 的消息
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   *    keywordListMatchType: 'and',
   * });
   *
   * @example
   * // 全量搜索，指定关键字列表和消息发送者搜索
   * // - 搜索消息发送者为 ‘user1’ 或 ‘user2’，且消息里出现了 ‘你好’ 或 ‘在哪里’ 的消息
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   *    senderUserIDList: ['user1', 'user2'],
   * });
   *
   * @example
   * // 全量搜索，指定关键字和消息类型搜索
   * // - 搜索消息类型为 ‘文本消息’ 或 ‘自定义消息’，且消息里出现了 ‘你好’ 或 ‘在哪里’ 的消息
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   *    messageTypeList: [TUIChatEngine.TYPES.MSG_TEXT, TUIChatEngine.TYPES.MSG_CUSTOM],
   * });
   *
   * @example
   * // 全量搜索，搜索所有图片消息（当指定发送者或消息类型时，关键字列表可以为空）
   * let promise = TUIChatService.searchCloudMessages({
   *    messageTypeList: [TUIChatEngine.TYPES.MSG_IMAGE],
   * });
   *
   * @example
   * // 全量搜索，只指定消息发送者和消息类型搜索（当指定发送者或消息类型时，关键字列表可以为空）
   * // - 搜索消息发送者为 ‘user1’ 或 ‘user2’，且消息类型为 ‘文本消息’ 或 ‘自定义消息’ 的消息
   * let promise = TUIChatService.searchCloudMessages({
   *    senderUserIDList: ['user1', 'user2'],
   *    messageTypeList: [TUIChatEngine.TYPES.MSG_TEXT, TUIChatEngine.TYPES.MSG_CUSTOM],
   * });
   *
   * @example
   * // 全量搜索，指定关键字和时间（过去一天）搜索
   * // - 搜索过去一天，消息里出现了 ‘你好’ 或 ‘在哪里’的消息
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   *    timePosition: Number((new Date().getTime()/1000).toFixed(0)),
   *    timePeriod: 24 * 60 * 60,
   * });
   * promise.then(function(imResponse) {
   *  // 搜索消息成功
   *    const { totalCount, cursor，searchResultList } = imResponse.data;
   *    console.log(totalCount); // 满足搜索条件的消息所在的所有会话总数量
   *    console.log(cursor); // 下一次云端搜索的起始位置, 如果没有表示搜索结果拉取完成
   *    console.log(searchResultList); // 满足搜索条件的消息根据会话 ID 分组，分页返回分组结果
   *    for (let i = 0; i < searchResultList.length; i++) {
   *       const searchResultItem = searchResultList[i];
   *       const { conversationID, messageCount, messageList } = searchResultItem;
   *       console.log(conversationID); // 会话 ID
   *       console.log(messageCount); // 当前会话一共搜索到了多少条符合要求的消息
   *       // 本次搜索【全部会话】，那么 messageList 中装载的消息条数会有如下两种可能：
   *       // - 如果某个会话中匹配到的消息条数 > 1，则 messageList 为空，您可以在 UI 上显示“ messageCount 条相关记录”。
   *       // - 如果某个会话中匹配到的消息条数 = 1，则 messageList 为匹配到的那条消息，您可以在 UI 上显示之，并高亮匹配关键词。
   *       console.log(messageList);
   *     }
   * }).catch(function(imError) {
   *    console.error(imError); // 搜索消息失败
   * });
   *
   * @example
   * // 指定会话，指定关键字搜索
   * // - 搜索在 'GROUPPublic001' 会话中，消息里出现 ‘你好’ 或 ‘在哪里’ 的消息。
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   *    conversationID: 'GROUPPublic001'
   * });
   *
   * @example
   * // 指定会话，指定消息类型搜索（当指定发送者或消息类型时，关键字列表可以为空）
   * // - 在 'GROUPPublic001' 会话中，搜索图片消息。
   * let promise = TUIChatService.searchCloudMessages({
   *    conversationID: 'GROUPPublic001',
   *    messageTypeList: [TUIChatEngine.TYPES.MSG_IMAGE],
   * });
   *
   * @example
   * // 指定会话，指定关键字和消息类型搜索
   * // - 搜索在 'GROUPPublic001' 会话中，消息里出现 ‘你好’ 或 ‘在哪里’，且消息类型为 ‘文本消息’ 或 ’自定义消息’ 的消息。
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   *    conversationID: 'GROUPPublic001'
   *    messageTypeList: [TUIChatEngine.TYPES.MSG_TEXT, TUIChatEngine.TYPES.MSG_CUSTOM],
   * });
   *
   * @example
   * // 指定会话，指定关键字和时间（过去一天）搜索
   * // - 搜索过去一天，在 'GROUPPublic001' 会话中，消息里出现 ‘你好’ 或 ‘在哪里’ 的消息。
   * let promise = TUIChatService.searchCloudMessages({
   *    keywordList: ['你好', '在哪里'],
   *    conversationID: 'GROUPPublic001'
   *    timePosition: Number((new Date().getTime()/1000).toFixed(0)),
   *    timePeriod: 24 * 60 * 60,
   * });
   * promise.then(function(imResponse) {
   *    // 搜索消息成功
   *    const { totalCount, cursor, searchResultList } = imResponse.data;
   *    console.log(totalCount); // 满足搜索条件的消息总数量
   *    console.log(cursor); // 下一次云端搜索的起始位置, 如果没有表示搜索结果拉取完成
   *    console.log(searchResultList); // 当前会话搜索的消息结果
   *    const { conversationID, messageCount, messageList } = searchResultList[0];
   *    console.log(conversationID); // 会话ID
   *    console.log(messageCount); // 当前会话一共搜索到了多少条符合要求的消息
   *    console.log(messageList); // 本会话中所有满足搜索条件的消息列表
   * }).catch(function(imError); {
   *    console.error(imError); // 搜索消息失败
   * });
   */
  searchCloudMessages(options: SearchCloudMessagesParams): Promise<any>;

  /**
   * 添加消息回应
   * @function
   * @param {IMessageModel} message 消息实实例 Model
   * @param {string} reactionID 消息回应 ID
   * @example
   * let promise = TUIChatService.addMessageReaction(message, 'smile');
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  addMessageReaction(message: IMessageModel, reactionID: string): Promise<any>;

  /**
   * 删除消息回应
   * @function
   * @param {IMessageModel} message 消息实实例 Model
   * @param {string} reactionID 消息回应 ID
   * @example
   * let promise = TUIChatService.removeMessageReaction(message, 'smile');
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  removeMessageReaction(message: IMessageModel, reactionID: string): Promise<any>;

  /**
   * 批量拉取多条消息回应信息
   * @function
   * @param {GetMessageReactionsParams} options 批量拉取多条消息回应信息参数
   * @private
   */
  getMessageReactions(options: GetMessageReactionsParams): void;

  /**
   * 分页拉取指定消息回应的用户列表
   * @function
   * @param {GetAllUserListOfMessageReactionParams} options 分页拉取指定消息回应的用户列表参数
   * @example
   * let promise = TUIChatService.getAllUserListOfMessageReaction({
   *   message: message,
   *   reactionID: 'smile',
   *   nextSeq: 0,
   *   count: 100,
   * });
   * promise.then(function(response) {
   *   // 获取成功
   *   const { userList, nextSeq, isCompleted } = response.data;
   *   // userList - 用户信息列表，包含用户的 userID、nick、avatar 信息
   *   // nextSeq - 用于续拉，分页续拉时需传入该字段
   *   // isCompleted - 表示是否已经拉完所有的用户信息。isCompleted 为 true 时，nextSeq 为 0
   * }).catch(function(error) {
   *   // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  getAllUserListOfMessageReaction(options: GetAllUserListOfMessageReactionParams): Promise<any>;

  /**
   * 语音转文字
   * @function
   * @param {ConvertVoiceToTextParams} options 语音转文字参数信息
   * @example
   * // 将语音转为中文
   * let promise = TUIChatService.convertVoiceToText({
   *  message: audioMessageModel,
   * });
   * promise.then((chatResponse) => {
   *  const { result } = chatResponse.data;
   * });
   * @example
   * // 将语音转为日文
   * let promise = TUIChatService.convertVoiceToText({
   *  message: audioMessageModel,
   *  language: 'ja-JP',
   * });
   * promise.then((chatResponse) => {
   *  const { result } = chatResponse.data;
   * });
   */
  convertVoiceToText(options: ConvertVoiceToTextParams): Promise<any>;

  /**
   * 清空单聊或群聊本地及云端的消息（不删除会话）
   * @function
   * @param {conversationID} string 会话 ID
   * @example
   * // 清空群聊 groupID 为 test 的历史消息
   * let promise = TUIChatService.clearHistoryMessage('GROUPtest');
   * promise.then((chatResponse) => {
   *  const { result } = chatResponse.data;
   * });
   */
  clearHistoryMessage(conversationID: string): Promise<any>;

  /**
   * 更新消息结构体的信息
   * @function
   * @param {Record<string, any>} options 用户昵称信息
   * @private
   */
  updateMessageInfo(options: Record<string, any>): void;
}
