import type {
  AddFriendParams,
  DeleteFriendParams,
  CheckFriendParams,
  GetFriendProfileParams,
  UpdateFriendParams,
  AcceptFriendApplicationParams,
  DeleteFriendApplicationParams,
} from '../../type';

/**
 * @interface TUIFriendService
*/
export interface ITUIFriendService {
  /**
   * 初始化 Service
   * @function
   * @private
   */
  init(): void;

  /**
   * 获取好友列表
   * @function
   * @private
   */
  getFriendList(): Promise<any>;

  /**
   * 获取好友备注
   * @function
   * @private
   */
  getFriendRemark(userIDList: string[]): Record<string, any>;

  /**
   * 添加好友
   * @function
   * @param {AddFriendParams} options 参数选项
   * @example
   * let promise = TUIFriendService.addFriend({
   *   to: 'user1',
   *   source: 'AddSource_Type_Web',
   *   remark: '小橙子',
   *   groupName: '好友',
   *   wording: '我是 user0',
   *   type: TUIChatEngine.TYPES.SNS_ADD_TYPE_BOTH,
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  addFriend(options: AddFriendParams): Promise<any>;

  /**
   * 删除好友
   * @function
   * @param {DeleteFriendParams} options 参数选项
   * @example
   * let promise = TUIFriendService.deleteFriend({
   *   userIDList: ['user1','user2'],
   *   type: TUIChatEngine.TYPES.SNS_DELETE_TYPE_BOTH
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  deleteFriend(options: DeleteFriendParams): Promise<any>;

  /**
   * 校验好友关系
   * @function
   * @param {CheckFriendParams} options 参数选项
   * @example
   * let promise = TUIFriendService.checkFriend({
   *   userIDList: ['user1','user2'],
   *   type: TUIChatEngine.TYPES.SNS_CHECK_TYPE_BOTH
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  checkFriend(options: CheckFriendParams): Promise<any>;

  /**
   *
   * 获取指定好友的全量好友数据（包含好友标配字段和好友自定义字段）和全量资料数据（包含资料标配字段和资料自定义字段）。
   * @function
   * @param {GetFriendProfileParams} options 参数选项
   * @example
   * let promise = TUIFriendService.getFriendProfile({
   *   userIDList: ['user1','user2']
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  getFriendProfile(options: GetFriendProfileParams): Promise<any>;

  /**
   * 更新好友的关系链数据，只支持好友备注、关系链自定义字段的更新操作。
   * @function
   * @param {UpdateFriendParams} options 参数选项
   * @example
   * // 更新好友备注
   * let promise = TUIFriendService.updateFriend({
   *   userID: 'user1',
   *   remark: 'aallen'
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   * @example
   * // 更新好友自定义字段
   * let promise = TUIFriendService.updateFriend({
   *   userID: 'user1',
   *   friendCustomField: [
   *    // 好友自定义字段必须以 Tag_SNS_Custom 为前缀
   *    { key: 'Tag_SNS_Custom_test1', value: 'hello' },
   *    { key: 'Tag_SNS_Custom_test2', value: 'world' },
   *   ]
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  updateFriend(options: UpdateFriendParams): Promise<any>;

  /**
   * 同意好友申请
   * @function
   * @param {AcceptFriendApplicationParams} options 参数选项
   * @example
   * let promise = TUIFriendService.acceptFriendApplication({
   *    userID: 'user1',
   *    remark: '客服小亮',
   *    type: TUIChatEngine.TYPES.SNS_APPLICATION_AGREE_AND_ADD
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  acceptFriendApplication(options: AcceptFriendApplicationParams): Promise<any>;

  /**
   * 拒绝好友申请
   * @function
   * @param {string} userID 被拒绝用户的 userID
   * @example
   * let promise = TUIFriendService.refuseFriendApplication('user1');
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  refuseFriendApplication(userID: string): Promise<any>;

  /**
   * 删除好友申请
   * @function
   * @param {DeleteFriendApplicationParams} options 参数选项
   * @example
   * let promise = TUIFriendService.deleteFriendApplication({
   *   userID: 'user1',
   *   type: TUIChatEngine.TYPES.SNS_APPLICATION_SENT_TO_ME
   * });
   * promise.catch((error) => {
   *  // 调用异常时业务侧可以通过 promise.catch 捕获异常进行错误处理
   * });
   */
  deleteFriendApplication(options: DeleteFriendApplicationParams): Promise<any>;

  /**
   * 上报好友申请已读
   * @function
   */
  setFriendApplicationRead(): Promise<any>;
}
