import TUIBase from '../tui-base';
import type { IConversationModel } from '../interface/model';
import { isPrivateKey, isUrl, calculateTimeAgo, JSONToObject } from '../utils/common-utils';
import type { MuteConversationParams } from '../type';
import { AVATAR, BASIC_EMOJI_NAME_MAPPING } from '../config';
import isEmpty from '../utils/is-empty';
import { IN_WX_MINI_APP } from '../utils/env';

const LAST_MSG_FOR_SHOW = [
  '[图片]',
  '[语音]',
  '[视频]',
  '[文件]',
  '[位置]',
  '[地理位置]',
  '[动画表情]',
  '[自定义消息]',
  '[群提示消息]',
  '[聊天记录]',
];

export default class ConversationModel extends TUIBase implements IConversationModel {
  public conversationID!: string;
  public type!: string;
  public subType!: string;
  public unreadCount!: number;
  public lastMessage!: {
    nick: string;
    nameCard: string;
    lastTime: number;
    lastSequence: string;
    fromAccount: string;
    isRevoked: boolean;
    revoker?: string | undefined;
    isPeerRead: boolean;
    messageForShow: string;
    type: string;
    payload: any;
  };

  public groupProfile?: any;
  public userProfile?: any;
  public groupAtInfoList!: any[];
  public remark!: string;
  public isPinned!: boolean;
  public messageRemindType!: string;
  public markList!: string[];
  public customData!: string;
  public conversationGroupList!: any[];
  public draftText!: string;
  public isMuted!: boolean;
  public operationType!: number;
  public _conversation: any;

  constructor(options: any) {
    super();
    this.initProxy(options);
    this.isMuted = (this.messageRemindType === this.getEngine().TYPES.MSG_REMIND_ACPT_NOT_NOTE)
    || (this.messageRemindType === this.getEngine().TYPES.MSG_REMIND_DISCARD);
    this.operationType = 0;
    this._conversation = options;
  }

  /**
   * 初始化挂载 conversation 字段
   */
  private initProxy(options: any) {
    Object.keys(options).forEach((key) => {
      if (!isPrivateKey(key)) {
        (this as any)[key] = options[key];
      }
    });
  }

  updateProperties(options: any) {
    Object.keys(options).forEach((key) => {
      if (!isPrivateKey(key)) {
        (this as any)[key] = options[key];
      }
    });
  }

  updateOperationType(operationType: number) {
    this.operationType = operationType;
  }

  getConversation() {
    return this._conversation;
  }

  deleteConversation() {
    return this.getEngine().TUIConversation.deleteConversation(this.conversationID);
  }

  pinConversation() {
    return this.getEngine().TUIConversation.pinConversation({
      conversationID: this.conversationID,
      isPinned: !this.isPinned,
    });
  }

  muteConversation() {
    const chatEngine = this.getEngine();
    const messageRemindType = (this.isMuted === true)
      ? chatEngine.TYPES.MSG_REMIND_ACPT_AND_NOTE
      : chatEngine.TYPES.MSG_REMIND_ACPT_NOT_NOTE;
    const options: MuteConversationParams = {
      messageRemindType,
    };
    if (this.type === chatEngine.TYPES.CONV_C2C) {
      const userID = this.conversationID.replace(chatEngine.TYPES.CONV_C2C, '');
      options.userIDList = [userID];
    } else if (this.type === chatEngine.TYPES.CONV_GROUP) {
      const groupID = this.conversationID.replace(chatEngine.TYPES.CONV_GROUP, '');
      options.groupID = groupID;
    }
    return chatEngine.TUIConversation.muteConversation(options);
  }

  // 以下方法用于处理 conversation 需要展示的数据
  getAvatar() {
    const chatEngine = this.getEngine();
    let avatar = '';
    switch (this.type) {
      case chatEngine.TYPES.CONV_C2C:
        avatar = isUrl(this.userProfile?.avatar) ? this.userProfile?.avatar : AVATAR.C2C;
        break;
      case chatEngine.TYPES.CONV_GROUP:
        avatar = isUrl(this.groupProfile?.avatar) ? this.groupProfile?.avatar : AVATAR.GROUP;
        break;
      case chatEngine.TYPES.CONV_SYSTEM:
        avatar = isUrl(this.groupProfile?.avatar) ? this.groupProfile?.avatar : AVATAR.SYSTEM;
        break;
      default:
        break;
    }
    return avatar;
  }

  getShowName() {
    const chatEngine = this.getEngine();
    let name = '';
    switch (this.type) {
      case chatEngine.TYPES.CONV_C2C:
        name = this.remark || this.userProfile?.nick || this.userProfile?.userID || '';
        break;
      case chatEngine.TYPES.CONV_GROUP:
        name = this.groupProfile?.name || this.groupProfile?.groupID || '';
        break;
      case chatEngine.TYPES.CONV_SYSTEM:
        name = chatEngine.TUITranslate.t('系统通知');
        break;
      default:
        break;
    }
    return name;
  }

  getGroupAtInfo() {
    const chatEngine = this.getEngine();
    const t = chatEngine.TUITranslate.t.bind(chatEngine.TUITranslate);
    const List: string[] = [
      `[${t('TUIConversation.有人@我')}]`,
      `[${t('TUIConversation.@所有人')}]`,
      `[${t('TUIConversation.@所有人')}][${t('TUIConversation.有人@我')}]`,
    ];
    let atInfo = '';
    for (let index = 0; index < this.groupAtInfoList.length; index++) {
      if (this.groupAtInfoList[index].atTypeArray[0] && this.unreadCount > 0) {
        atInfo = List[this.groupAtInfoList[index].atTypeArray[0] - 1];
      }
    }
    return atInfo;
  }

  getLastMessage(key: string) {
    if (key === 'time') {
      return this.getLastMessageTime();
    }
    if (key === 'text') {
      return this.getLastMessageText();
    }
    console.warn(`ConversationModel.getLastMessage key:${key} is invalid.`);
    return null;
  }

  private getLastMessageTime() {
    const chatEngine = this.getEngine();
    let time = '';
    if (this.lastMessage?.lastTime > 0) {
      time = calculateTimeAgo(
        this.lastMessage.lastTime * 1000,
        chatEngine.TUITranslate.t.bind(chatEngine.TUITranslate),
      );
    }
    return time;
  }

  private getLastMessageText() {
    // 会话草稿存在，返回草稿内容
    if (this.draftText) {
      const draftInfo = JSONToObject(this.draftText);
      return draftInfo.abstract;
    }
    const chatEngine = this.getEngine();
    const t = chatEngine.TUITranslate.t.bind(chatEngine.TUITranslate);
    if (this.operationType === 4) {
      return t('TUIConversation.您已被群管理员移出群聊');
    }
    if (this.operationType === 5) {
      return t('TUIConversation.该群聊已被解散');
    }
    if (this.operationType === 8) {
      return t('TUIConversation.您已退出该群聊');
    }
    const conversation = this._conversation;
    const { lastMessage } = this;
    let lastMessageSender = '';
    let lastMessagePayload: any = '';
    // Judge the number of unread messages and display
    //   them only when the message is enabled without interruption.
    const showUnreadCount = conversation.unreadCount > 0
      && conversation.messageRemindType === chatEngine.TYPES.MSG_REMIND_ACPT_NOT_NOTE
      ? `[${conversation.unreadCount > 99 ? '99+' : conversation.unreadCount}${t('TUIConversation.条')}]`
      : '';
    // Determine the lastmessage sender of the group.
    // Namecard / Nick / userid is displayed by priority
    if (conversation.type === chatEngine.TYPES.CONV_GROUP) {
      if (lastMessage.fromAccount === chatEngine.getMyUserID()) {
        lastMessageSender = t('TUIConversation.我');
      } else {
        // 群会话：remark > nameCard > nick > userID (对齐微信体验)
        const userID: string = lastMessage.fromAccount;
        const friendRemark: any = chatEngine.TUIFriend.getFriendRemark([userID]);
        lastMessageSender = friendRemark[userID]
        || lastMessage.nameCard
        || lastMessage.nick
        || userID;
      }
    }
    // Display content of lastmessage message body
    if (lastMessage.type === chatEngine.TYPES.MSG_TEXT) {
      lastMessagePayload = this.decodeTextMessage(lastMessage.payload.text);
    } else if (lastMessage.type === chatEngine.TYPES.MSG_CUSTOM) {
      const data = JSONToObject(lastMessage?.payload?.data);
      if (data?.businessID === 1) {
        lastMessagePayload = chatEngine.TUIChat.messageHandler.handleCallKitSignaling(lastMessage);
        return lastMessagePayload;
      }
      lastMessagePayload = LAST_MSG_FOR_SHOW.includes(lastMessage.messageForShow) ? t(`TUIChat.${lastMessage.messageForShow}`) : lastMessage.messageForShow;
    } else {
      lastMessagePayload = LAST_MSG_FOR_SHOW.includes(lastMessage.messageForShow) ? t(`TUIChat.${lastMessage.messageForShow}`) : lastMessage.messageForShow;
    }

    if (lastMessage.isRevoked) {
      lastMessagePayload = t('TUIChat.撤回了一条消息');
    }
    if (conversation.type === chatEngine.TYPES.CONV_GROUP
      && lastMessage.type === chatEngine.TYPES.MSG_GRP_TIP) {
      return lastMessagePayload;
    }
    // Specific display content of message box
    return `${showUnreadCount}${lastMessageSender ? `${lastMessageSender}:` : ''}${lastMessagePayload}`;
  }

  private decodeTextMessage(text: string) {
    if (isEmpty(text)) {
      return text;
    }
    const reg = /(\[.+?\])/g;
    let txt: string = text;
    if (reg.test(text)) {
      const chatEngine = this.getEngine();
      const t = chatEngine.TUITranslate.t.bind(chatEngine.TUITranslate);
      txt = text.replace(reg, (match) => {
        const ret = t(`Emoji.${match}`);
        // v2.2.0 起支持 emoji 国际化
        // uni 打包微信小程序或低版本 TUIKit 升级新版本 engine 时此处需要做兼容
        if (IN_WX_MINI_APP || !ret || ret.startsWith('Emoji.')) {
          return BASIC_EMOJI_NAME_MAPPING[match] || match;
        }
        return ret;
      });
    }
    return txt;
  }
}
